@@ -118,11 +118,11 @@ class LinkDir(UserDict):
   def writeData(self, fw, nextPfx):
     if self.hierarchical:
       for lnk in self.values():
-        fw.write(nextPfx+"addChild "+lnk.son+'\n\t\t\t' + lnk.parent+ '\n')
+        fw.write(f"{nextPfx}addChild {lnk.son}\n\t\t\t{lnk.parent}\n")
     else: 
       for lnk in self.values():
         for otherName in lnk.parent:
-          fw.write(nextPfx+"weld "+lnk.son+'\n\t\t\t'+otherName+'\n')
+          fw.write(f"{nextPfx}weld {lnk.son}\n\t\t\t{otherName}\n")
 
 
 
@@ -1408,6 +1408,47 @@ class Figure():
     #Wend
     return C_OK
 
+
+  def clearAllChannel(self, pattern, lstActors=None, hide=True):
+    ''' Clear (delete morphs) in all channels with a name that starts with the pattern.
+    Search limited to a set of actors, if lstActors is not null
+    
+    
+    Returns
+    -------
+    int
+      C_OK : Success
+      C_FAIL : Actor not found or Missing target channel in some descendant
+    '''
+    # Build the list of actors
+    if lstActors:
+      lstSrcActors = []
+      
+      for actId in lstActors:
+        a = self.findActor(actId) if isinstance(actId, str) else actId 
+        if not a:
+          logging.warning('Actor(%s) not found', actId)
+          return C_FAIL
+        lstSrcActors.append(a)
+    else:
+      lstSrcActors = self.getActors()    
+    
+    # Build the list of all targetGeom that matches the pattern
+    lstgt = [ gt for pa in lstSrcActors for gt in pa.getChannels()  \
+              if (gt.getPoserType()==PoserToken.E_targetGeom)  \
+              and gt.getName().startswith(pattern) ]
+
+    for gt in lstgt:
+      logging.info(f"Channel {gt.getName()} cleared in {gt.getPoserMeshedObject().getName()}")      
+      gt.removeDeltas()
+      if hide:
+        gt.setHidden(True)
+    #End for
+
+
+    return C_OK
+
+
   # 
   # @param pattern
   def cleanMagnetMaterial(self):
