@@ -101,7 +101,13 @@ def index(n):
     return -1
   c = n.rfind(':')
   return int(n[c + 1:]) if (c >= 0) else 0
- 
+
+def chgIndex(s,ni):
+  ''' Replace a potential figure index by a new one.
+  Set new one if none
+  '''
+  pi = s.rfind(':')
+  return f'{s}:{ni}' if pi<0 else f'{s[:pi]}:{ni}'  
  
 def nodeNameNo(n):
   '''
