@@ -1089,9 +1089,9 @@ class PoserFile(object):
 
 
 
-  def deleteFigureImpact(self, fig):
-    logging.error("Not implemented yet")
-    return None
+#   def deleteFigureImpact(self, fig):
+#     logging.error("Not implemented yet")
+#     return None
 
   def delete(self, obj):
     '''  Delete the given PoserProp or Figure from the list of general props '''
