@@ -443,7 +443,7 @@ class Node(StructuredAttribut):
 
 
   def setLocation(self, x, y):
-    self._pos = str(x)+ ' ' + str(y)
+    self._pos = f'{x} {y}'
 
   def getNodeType(self):
     return PoserConst.kNodeTypeCodeBLENDER if self._typeName=='blender' else PoserConst.kNodeTypeCodeIMAGEMAP if self._typeName=='image_map' else C_ERROR
@@ -457,7 +457,7 @@ class ShaderTree(StructuredAttribut):
     self.setPoserType(PoserToken.E_shaderTree)
 
   def getNodeByInternalName(self, intName):
-    return next( (n for n in self.getLstAttr() if isinstance(n, Node) and n._nodeName==intName), None)
+    return next( (n for n in self.getLstAttr() if isinstance(n, Node) and ((n._nodeName==intName) or (n._name==intName))), None)
 
   def getNodeByName(self, name):
     return next( (n for n in self.getLstAttr() if isinstance(n, Node) and n.getName()==name), None)
@@ -1351,21 +1351,20 @@ class GenericTransform(StructuredAttribut):
   def ishasDeltas(self): return self._dlt and (len(self._dlt.deltaSet)>0)
 
   # 
-  #    * Remove the deltas of tha current channel
-  #    
   def removeDeltas(self):
+    ''' Remove the deltas of tha current channel.
+    Do not delete anymore the calculation dependencies
+    '''
     #  20090711 : Not sure that should be REALLY cleared?
     #  As of 20100226 do not delete anymore the calculation dependencies : _lstVOD.clear();
-    #self._indexes = 0
     self._numbDeltas = 0
     #  20190105 : Fix for robusness
     if self.ishasDeltas():
       self.getDeltas().deltaSet.clear()
 
   # 
-  # Optimize deltas : Delete delta with a too small norme
-  #    
   def optimizeDeltas(self, refNorm):
+    ''' Optimize deltas : Delete delta with a too small norm.    '''
     dl = self.getDeltas()
     oldLength = len(dl.deltaSet)
     optSet = { no:t for no,t in dl.deltaSet.items() if t.norme() > refNorm }
