@@ -20,14 +20,21 @@ Created on 23 nov. 2020
 '''
 import logging
 
-from pypos3d.wftk.WFBasic import Point3d, TexCoord2f, C_FACE_TEXT
+from langutil import C_FAIL
+from pypos3d.wftk.WFBasic import Point3d, TexCoord2f, C_FACE_TEXT, FEPSILON3
 from pypos3d.wftk.WaveGeom import WaveGeom
-
 from pypos3d.propslim import MX_UNBOUND, MX_ALL_MASK, MX_VALID_FLAG, MX_PERVERTEX, MX_MAX_BINDING
 
 
 class MxFace: # implements Blockable
-  #public/* MxVertexID */int v[] = new int[3]
+  # Enhancement: Used sloted class for performance aspect
+  v0:int
+  v1:int
+  v2:int
+  mark:int
+  tag:int
+  grpNo:int # Original groupe id
+  matIdx:int # Preserve original material index
 
   def __init__(self, v0, v1, v2, matIdx, grpNo):
     ''' Constructors: -1 supposed not used for real vertex indexes
@@ -96,20 +103,33 @@ class MxStdModel(object):
     
   #public/* MxFaceID */int add_face(int v1, int v2, int v3, boolean will_link /* =True */)
   def add_face(self, v1, v2, v3, isProtected, matIdx, grpNo):
-    f = MxFace(v1, v2, v3, matIdx, grpNo)
-    self.faces.append(f)
-    vid = len(self.faces)-1
-
-    # We always link if will_link: # self.init_face(vid)
-    self.face_links[v1].append(vid)
-    self.face_links[v2].append(vid)
-    self.face_links[v3].append(vid)
-
-    if isProtected:
-      self.protvert.add(v1)
-      self.protvert.add(v2)
-      self.protvert.add(v3)
-
+    # FIX: Check Triangle surface before adding it
+    # Compute each edges length
+    cp0,cp1,cp2 = self.vertices[v1], self.vertices[v2], self.vertices[v3]
+    l0,l1,l2 = cp0.distance(cp1), cp1.distance(cp2), cp2.distance(cp0)
+      
+    # Compute Surface (Heron formula)
+    # Protect from rounding effects that create very small negative s2
+    p = 0.5*(l0+l1+l2)
+    s2 = p*(p-l0)*(p-l1)*(p-l2) # s2 is the Squared surface of the triangle
+    if s2<FEPSILON3: # Rejected Triangle      
+      logging.warning("Triangle[%d, %d, %d] rejected: too small (%g)", v1, v2, v3, s2)
+      return C_FAIL
+    else:
+      f = MxFace(v1, v2, v3, matIdx, grpNo)
+      self.faces.append(f)
+      vid = len(self.faces)-1
+  
+      # We always link if will_link: # self.init_face(vid)
+      self.face_links[v1].append(vid)
+      self.face_links[v2].append(vid)
+      self.face_links[v3].append(vid)
+  
+      if isProtected:
+        self.protvert.add(v1)
+        self.protvert.add(v2)
+        self.protvert.add(v3)
+      
     return vid
 
   def add_texcoord(self, s, t):
