@@ -3,7 +3,6 @@
 import sys, copy
 import logging
 import math
-import collections
 import array
 import numpy as np
 from scipy import spatial
@@ -12,7 +11,8 @@ from langutil import C_FAIL, C_ERROR , C_OK
 
 from pypos3d.wftk import WFBasic
 from pypos3d.wftk.WFBasic import C_VERTEX_NOT_FOUND, COUNT, FEPSILON, C_FACE_NORM, \
-  C_FACE_TEXT, TexCoord2f, CoordSyst, C_FACE_MATMASK, C_FACE_TEXTNORM
+  C_FACE_TEXT, TexCoord2f, CoordSyst, C_FACE_MATMASK, C_FACE_TEXTNORM, FEPSILON2, C_FACE_SURF, \
+  C_FACE_ORDER_ANGLE, C_FACE_ORDER_LENGTH, C_FACE_ORDER_SURF, C_FACE_NONE
 from pypos3d.wftk.WFBasic import LowestIdxPos, CreateLoop, FaceNormalOrder
 from pypos3d.wftk.WFBasic import Point3d, Vector3d, Edge, Regularity, IndexAdd, CreateTransform, MxProd
 from pypos3d.wftk.PaveList2D import PaveList2D
@@ -104,7 +104,7 @@ class GeomGroup(object):
     self.lstMat = src.lstMat if src else None
 
   def __str__(self):
-    return 'GeomGroup[{0:s}]: {1:d} faces'.format(self._name, self.getNbFace())
+    return f'GeomGroup[{self._name}]: {self.getNbFace()} faces'
 
 
   def getNbFace(self):
@@ -1594,15 +1594,16 @@ class GeomGroup(object):
       ret = C_FAIL
             
     return ret, vr
-    
-      
-  
-  def unTriangularize(self, maxsin=FEPSILON):
+
+
+  def unTriangularize(self, maxsin=FEPSILON, algo=C_FACE_NONE, surfaceThreshold=0.1):
     ''' Untriangularize the current group (inplace).
     Triangular faces are merged when:
     * They have the same material
     * They have a common edge
-    * The sinus of their normals (angle) is smaller than 'maxsin'
+    * The sinus of their normals (angle) is smaller than 'maxsin' and 
+      when 'algo' parameter is C_FACE_SURF if the merged face represents more than 'surfaceThreshold' percent
+      of the final face.
     
     Parameters
     ----------
@@ -1610,6 +1611,20 @@ class GeomGroup(object):
       Maximal value for the sinus of the angle of both
       normals to permit a face merge
       
+    algo: bitmask for algorithm selection, default = C_FACE_NONE
+      Add C_FACE_SURF for relative surface filtering. 
+      
+      Add either C_FACE_ORDER_LENGTH, C_FACE_ORDER_ANGLE or C_FACE_ORDER_SURF for surface choice ordering
+      The ordering is used to choose the merged triangle between the 3 neighbours of a destination triangle.
+      C_FACE_ORDER_LENGTH: The triangle with the greastest common edge is merged
+      C_FACE_ORDER_ANGLE: The triangle with the smallest angle is merged
+      C_FACE_ORDER_SURF: The triangle with the greatest relative surface is merged
+      
+    surfaceThreshold: float, default=0.1
+      Threshold used by the surface filtering to keep a triangle as a merging candidate.
+      The candidat is kept is its surface will represent more than 'surfaceThreshold' of the
+      merged quad surface.
+      
     Return
     ------
     int : 
@@ -1623,70 +1638,80 @@ class GeomGroup(object):
     nStrip = [0, ]
     initNbFace = self.getNbFace()
     
+    # Initialize a list of list of triangles indexed by the vertex Id
+    vertexId2Tri = [ None ] * len(self.coordList)
+    
     for noface in range(0, initNbFace):
+       
       startIdx = self.getFaceStartIdx(noface)
       lastIdx = self.getFaceLastIdx(noface)
+      keepIt = (lastIdx-startIdx!=3)
       
-      if lastIdx-startIdx==3: # It's a triangle
-        
-        p0,p1,p2 = self.vertIdx[startIdx],self.vertIdx[startIdx+1], self.vertIdx[startIdx+2]           
-        vn = Point3d.triangle_normal(self.coordList[p0], self.coordList[p1], self.coordList[p2])
-        edges =  ( (p0,p1), (p1,p2), (p2,p0) )
-        revegdes = ( (p0,p2), (p2,p1), (p1,p0) )
-        
-        tf = TriFace(noface, p0, p1, p2, vn, edges, revegdes)
-        
-        lstTri.append(tf)
-        
-      else: # Just copy non triangular face
+      if not keepIt: # It's a triangle
+        # Create a triangular face with some pre-computed values
+        p0,p1,p2 = self.vertIdx[startIdx], self.vertIdx[startIdx+1], self.vertIdx[startIdx+2]
+        tf = TriFace(noface, p0, p1, p2, coordList=self.coordList)
+
+        if tf.surface > FEPSILON2: # Filter too small or null surface
+          lstTri.append(tf)
+          
+          for i in [p0, p1, p2]:
+            l = vertexId2Tri[i]
+            if l is None:
+              vertexId2Tri[i] = [ tf, ]
+            elif not tf in l:
+              l.append(tf)
+        else: # Triangular surface is too small, Store it
+          keepIt = True
+           
+      if keepIt: # Just copy non triangular face OR rejected triangles
         nVertIdx += self.vertIdx[startIdx:lastIdx]
         nStrip.append(len(nVertIdx))
         nMatIdx.append(self.matIdx[noface])
     
     dn2 = maxsin*maxsin
-    # Check triangle by 2 with a filter on the material index ... but it's an O(n2) algo
+    # Check triangle by 2 with a filter on the material index and according to the 'algo'
+    # a comparison of surfaces
     while lstTri:
       # Get the first triangle
       curTri = lstTri.pop()
-      e0 = curTri.edges[0]
-      e1 = curTri.edges[1]
-      e2 = curTri.edges[2]
+      p0, p1, p2 = curTri.p0, curTri.p1, curTri.p2
+      ltri0, ltri1, ltri2 = vertexId2Tri[p0], vertexId2Tri[p1], vertexId2Tri[p2]
+
+      ltri0.remove(curTri)
+      ltri1.remove(curTri)
+      ltri2.remove(curTri)
       
       fao = self.matIdx[curTri.noface]
       mi = fao.fAttr
       
-      # Compute the list of adjacent triangles (up to 3)
-      lstAdj = [ t for t in lstTri \
-                if (self.matIdx[t.noface].fAttr==mi) and \
-                  ( (e0 in t.revedges) or (e1 in t.revedges) or (e2 in t.revedges) ) and \
-                  (curTri.vnorm.cross(t.vnorm).norme2() <= dn2) ]
+      # Compute the list of candidate triangles:
+      lstCandidateTri = ltri0.copy() if ltri0 else []
+      lstCandidateTri += [ t for t in (ltri1 if ltri1 else []) if t not in lstCandidateTri ]
+      lstCandidateTri += [ t for t in (ltri2 if ltri2 else []) if t not in lstCandidateTri ]
       
+      # Compute the list of adjacent triangles (up to 3)
+      e0,e1,e2 = curTri.edges
+      lstAdj = [ t for t in lstCandidateTri \
+                  if (self.matIdx[t.noface].fAttr==mi) and \
+                    ( (e0 in t.revedges) or (e1 in t.revedges) or (e2 in t.revedges) ) and \
+                    (curTri.vnorm.cross(t.vnorm).norme2() <= dn2) ]
+      
+      if algo & C_FACE_SURF: # Add a filter on surface comparison
+        lstAdj = [ t for t in lstAdj if math.fabs(t.surface/(curTri.surface + t.surface)) >= surfaceThreshold ]
+        
       if lstAdj: # Choose the 'best' triangle to merge
-        adjTri = lstAdj[0]
-        
-        # Find the common Edge
-        ceCurTri = commEdge(curTri, adjTri)
-        lenEdge  = self.coordList[curTri.edges[ceCurTri][0]].distance( self.coordList[curTri.edges[ceCurTri][1]] )
-        
-        if len(lstAdj)>1:
-          ce = commEdge(curTri, lstAdj[1])
-          le = self.coordList[curTri.edges[ce][0]].distance( self.coordList[curTri.edges[ce][1]] )
-          if le > lenEdge:
-            adjTri = lstAdj[1]
-            ceCurTri = ce
-            
-        if len(lstAdj)>2:
-          ce = commEdge(curTri, lstAdj[2])
-          le = self.coordList[curTri.edges[ce][0]].distance( self.coordList[curTri.edges[ce][1]] )
-          if le > lenEdge:
-            adjTri = lstAdj[2]
-            ceCurTri = ce
+        adjTri, ceCurTri = curTri.choose(lstAdj, algo)
 
         # Remove Adjacent Triangle from the list of available faces
         lstTri.remove(adjTri)
+        
+        vertexId2Tri[adjTri.p0].remove(adjTri)
+        vertexId2Tri[adjTri.p1].remove(adjTri)
+        vertexId2Tri[adjTri.p2].remove(adjTri)
           
         # Merge curTri and adjTri
-        cea = commEdge(adjTri, curTri)
+        cea = adjTri.commEdge(curTri)
         pp = adjTri.p2 if cea==0 else (adjTri.p0 if cea==1 else adjTri.p1)
         if ceCurTri==0:
           tv = [ curTri.p0, pp, curTri.p1, curTri.p2 ]
@@ -1721,8 +1746,7 @@ class GeomGroup(object):
           else:
             fao.normIdx = array.array('l', [ fao.normIdx[0], fao.normIdx[0+1], fao.normIdx[0+2], np ])
         
-      else:
-        # The current triangle has no relevant adjacent, Add it to the new group
+      else: # The current triangle has no relevant adjacent, Add it to the new group
         startIdx = self.getFaceStartIdx(curTri.noface)
         lastIdx = self.getFaceLastIdx(curTri.noface)
         nVertIdx += self.vertIdx[startIdx:lastIdx]
@@ -1737,11 +1761,113 @@ class GeomGroup(object):
     self.matIdx[:] = nMatIdx
     
     return self.getNbFace() if initNbFace > self.getNbFace() else C_FAIL
+    
+      
   
-TriFace = collections.namedtuple('TriFace', ['noface', 'p0', 'p1', 'p2', 'vnorm', 'edges', 'revedges'])
+# Named tuple replaced by a 'sloted' class for both performance and extension
+#TriFace = collections.namedtuple('TriFace', ['noface', 'p0', 'p1', 'p2', 'vnorm', 'edges', 'revedges'])
 
-def commEdge(t, adjt):
-  return 0 if (t.p0,t.p1) in adjt.revedges else (1 if (t.p1,t.p2) in adjt.revedges else (2 if (t.p2, t.p0) in adjt.revedges else -1))
+class TriFace:
+  ''' This class represents a triangular face used during 'Untriangularization' algorithm.
+  '''
+  noface:int
+  p0:int
+  p1:int
+  p2:int
+  edges:tuple
+  revedges:tuple
+  vnorm:float
+  lenedges:tuple
+  surface:float
+  
+  def __init__(self, noface, p0, p1, p2, coordList=None):
+    
+    self.noface = noface
+    self.p0 = p0          
+    self.p1 = p1
+    self.p2 = p2
+              
+    self.edges =  ( (p0,p1), (p1,p2), (p2,p0) )
+    self.revedges = ( (p0,p2), (p2,p1), (p1,p0) )
+    
+    # Perform computations
+    if coordList:
+      cp0,cp1,cp2 = coordList[p0], coordList[p1], coordList[p2]
+      
+      # Compute Normale
+      self.vnorm = Point3d.triangle_normal(cp0, cp1, cp2)
+      
+      # Compute each edges length
+      l0,l1,l2 = cp0.distance(cp1), cp1.distance(cp2), cp2.distance(cp0)
+      self.lenedges = ( l0, l1, l2 )
+      
+      # Compute Surface (Heron formula)
+      # Protect from rounding effects that create very small negative s2
+      p = 0.5*(l0+l1+l2)
+      s2 = p*(p-l0)*(p-l1)*(p-l2)
+      self.surface = 0.0 if s2 < 0.0 else math.sqrt(s2) 
+      
+  def commEdge(self, adjt):
+    return 0 if (self.p0,self.p1) in adjt.revedges else \
+           (1 if (self.p1,self.p2) in adjt.revedges else \
+            (2 if (self.p2, self.p0) in adjt.revedges else -1))
+           
+  def choose(self, lstAdj, algo):
+    adjTri = lstAdj[0]
+    if algo & C_FACE_ORDER_ANGLE:
+      sinus2 = self.vnorm.cross(adjTri.vnorm).norme2()
+      
+      if len(lstAdj)>1:
+        ns = self.vnorm.cross(lstAdj[1].vnorm).norme2()
+        if ns < sinus2:
+          adjTri = lstAdj[1]
+          sinus2 = ns
+          
+      if len(lstAdj)>2:
+        ns = self.vnorm.cross(lstAdj[2].vnorm).norme2()
+        if ns < sinus2:
+          adjTri = lstAdj[2]
+
+      ceCurTri = self.commEdge(adjTri)
+      
+    elif algo & C_FACE_ORDER_SURF: #Keep the most contributing triangle
+      deltas = math.fabs( adjTri.surface / (adjTri.surface + self.surface ) )
+      
+      if len(lstAdj)>1:
+        nd = math.fabs( lstAdj[1].surface / (lstAdj[1].surface + self.surface))
+        if nd > deltas:
+          adjTri = lstAdj[1]
+          deltas= nd
+          
+      if len(lstAdj)>2:
+        nd = math.fabs( lstAdj[2].surface / (lstAdj[2].surface + self.surface))
+        if nd > deltas:
+          adjTri = lstAdj[2]
+
+      ceCurTri = self.commEdge(adjTri)
+
+    else: # algo & C_FACE_ORDER_LENGTH:    
+      # Find the common Edge
+      ceCurTri = self.commEdge(adjTri)
+      lenEdge  = self.lenedges[ceCurTri]
+      
+      if len(lstAdj)>1:
+        ce = self.commEdge(lstAdj[1])
+        le = self.lenedges[ce]
+        if le > lenEdge:
+          adjTri = lstAdj[1]
+          lenEdge = le
+          ceCurTri = ce
+          
+      if len(lstAdj)>2:
+        ce = self.commEdge(lstAdj[2])
+        le = self.lenedges[ce]
+        if le > lenEdge:
+          adjTri = lstAdj[2]
+          ceCurTri = ce
+      
+      
+    return adjTri, ceCurTri
 #
 
 
