@@ -731,7 +731,9 @@ class Vector3d(Point3d):
     
 
 class TexCoord2f():
-  ''' This class represent a (x,y) mutable tuple of float. '''
+  ''' This class represent a (x,y) mutable tuple of float used to
+      represent (U,V) coordinates of textured faces
+  '''
 
   def __init__(self, x=0.0, y=0.0):
     ''' Create a new TexCoord2f from various inputs.
@@ -739,7 +741,7 @@ class TexCoord2f():
     TexCoord2f() --> (0.0, 0.0)
     TexCoord2f(X,Y,Z as float) --> (X, Y)
     TexCoord2f(P as TexCoord2f) --> (p.x, p.y)
-    TexCoord2f([X,Y]) --> ([0], [1], [2])
+    TexCoord2f([X,Y]) --> ([0], [1])
 
     Parameters
     ----------
